@@ -10,16 +10,22 @@
 #' @section Multiple rows of column spanners \code{cgroup}:
 #'
 #' If you want to have a column spanner in multiple levels you can
-#' set the \code{cgroup} and \code{n.cgroup} arguments to matrices.
-#' If the different levels have different number of elements you
-#' need to set the ones that lack elements to NA. For instance
+#' set the \code{cgroup} and \code{n.cgroup} arguments to a \code{matrix} or
+#'  \code{list}.
+#'
+#' If the different levels have different number of elements and you have
+#' provided a **matrix** you need to set the ones that lack elements to NA. For instance
 #' \code{cgroup = rbind(c("first", "second", NA), c("a", "b", "c"))}.
 #' And the corresponding n,cgroup would be \code{n.cgroup = rbind(c(1, 2, NA), c(2, 1, 2))}.
 #' for a table consisting of 5 columns. The "first" spans the first two columns,
 #' the "second" spans the last three columns, "a" spans the first two, "b"
 #' the middle column, and "c" the last two columns.
 #'
-#' @section The \code{rgroup} arguement:
+#' It is recommended to use `list` as you will not have to bother with the `NA`.
+#'
+#' If you want leav a cgroup empty then simply provide `""` as the cgroup.
+#'
+#' @section The \code{rgroup} argument:
 #'
 #'  The rgroup allows you to smoothly group rows. Each row within a group
 #'  receives an indention of two blank spaces and are grouped with their
@@ -66,6 +72,15 @@
 #' In vignettes and other directly knitted documents you may need to either set
 #' \code{useViewer = FALSE} alternatively set \code{options(htmlTable.cat = TRUE)}.
 #'
+#' @section RStudio's notebook:
+#'
+#' RStudio has an interactive notebook that allows output directly into the document.
+#' In order for the output to be properly formatted it needs to have the \code{class}
+#' of \code{html}. The \code{htmlTable} tries to identify if the environment is a
+#' notebook document (uses the rstudio api and identifies if its a file with and `Rmd`
+#' file ending or if ther is an element with `html_notebook`). If you don't want this
+#' behaviour you can remove it using the `options(htmlTable.skip_notebook = TRUE)`
+#'
 #' @section Table counter:
 #'
 #' If you set the option table_counter you will get a Table 1,2,3
@@ -80,6 +95,24 @@
 #' for compatibility reasons. If you set \code{options(table_counter_roman = TRUE)}
 #' then the table counter will use Roman numumerals instead of Arabic.
 #'
+#'@section The \code{css.cell} argument:
+#'
+#' The \code{css.cell} parameter allows you to add any possible CSS style
+#' to your table cells.  \code{css.cell} can be either a vector or a matrix.
+#'
+#' If  \code{css.cell} is a \emph{vector}, it's assumed that the styles should be repeated
+#' throughout the rows (that is, each element in css.cell specifies the style
+#' for a whole column of 'x').
+#'
+#' In the case of  \code{css.cell} being a \emph{matrix} of the same size of the \code{x} argument,
+#' each element of \code{x} gets the style from the corresponding element in css.cell.  Additionally,
+#' the number of rows of \code{css.cell} can be \code{nrow(x) + 1} so the first row of of \code{css.cell}
+#' specifies the style for the header of \code{x}; also the number of columns of \code{css.cell}
+#' can be \code{ncol(x) + 1} to include the specification of style for row names of \code{x}.
+#'
+#' Note that the \code{text-align} CSS field in the \code{css.cell} argument will be overriden
+#' by the \code{align} argument.
+#'
 #'@section Empty dataframes:
 #' An empty dataframe will result in a warning and output an empty table, provided that
 #' rgroup and n.rgroup are not specified. All other row layout options will be ignored.
@@ -161,11 +194,11 @@
 #'  default so that each row group contains the same number of rows. If you want additional
 #'  rgroup column elements to the cells you can sett the "add" attribute to \code{rgroup} through
 #'  \code{attr(rgroup, "add")}, see below explaining section.
-#' @param cgroup A vector or a matrix of character strings defining major column header. The default
+#' @param cgroup A vector, matrix or list of character strings defining major column header. The default
 #'  is to have none. These elements are also known as \emph{column spanners}. If you want a column \emph{not}
 #'  to have a spanner then put that column as "". If you pass cgroup and \code{n.crgroup} as
 #'  matrices you can have column spanners for several rows. See cgroup section below for details.
-#' @param n.cgroup An integer vector or matrix containing the number of columns for which each element in
+#' @param n.cgroup An integer vector, matrix or list containing the number of columns for which each element in
 #'  cgroup is a heading. For example, specify \code{cgroup=c("Major_1","Major_2")},
 #'  \code{n.cgroup=c(3,3)} if \code{"Major_1"} is to span columns 1-3 and
 #'  \code{"Major_2"} is to span columns 4-6.
@@ -174,8 +207,10 @@
 #'  the number of columns in the matrix/data.frame then it automatically adds those.
 #' @param tspanner The table spanner is somewhat of a table header that
 #'  you can use when you want to join different tables with the same columns.
-#' @param n.tspanner An integer vector with the number of rows in the original matrix that
-#'  the table spanner should span.
+#' @param n.tspanner An integer vector with the number of rows or rgroups in the original
+#'  matrix that the table spanner should span. If you have provided one fewer n.tspanner elements
+#'  the last will be imputed from the number of rgroups (if you have provided `rgroup` and
+#'  `sum(n.tspanner) < length(rgroup)`) or the number of rows in the table.
 #' @param total The last row is sometimes a row total with a border on top and
 #'  bold fonts. Set this to \code{TRUE} if you are interested in such a row. If you
 #'  want a total row at the end of each table spanner you can set this to \code{"tspanner"}.
@@ -191,11 +226,7 @@
 #' @param css.tspanner.sep The line between different spanners
 #' @param css.total The css of the total row
 #' @param css.cell The css.cell element allows you to add any possible CSS style to your
-#'  table cells. If you provide a vector the vector it is assummed that the styles should
-#'  be repeated throughout the columns. If you provide a matrix of the same size as your
-#'  \code{x} argument. If have \code{ncol(x) + 1} the first row will correspond to the
-#'  rowname style. Correspondingly if the size is \code{nrow(x) + 1} it is assummed that the
-#'  first row is the header row.
+#'  table cells. See section below for details.
 #' @param css.class The html CSS class for the table. This allows directing html
 #'  formatting through \href{http://www.w3schools.com/Css/}{CSS}
 #'  directly at all instances of that class. \emph{Note:} unfortunately the
@@ -237,6 +268,10 @@
 #'  (at least that is how my 2010 version behaves). You can additinally use the
 #'  \code{options(htmlTableCompat = "html")} if you want a change to apply
 #'  to the entire document.
+#'  MS Excel sometimes misinterprets certain cell data when opening HTML-tables (eg. 1/2 becomes 1. February).
+#'  To avoid this please specify the correct Microsoft Office format for each cell in the table using the css.cell-argument.
+#'  To make MS Excel interpret everything as text use "mso-number-format:\"\\@\"".
+#' @param escape.html logical: should HTML characters be escaped? Defaults to FALSE.
 #' @return \code{string} Returns a string of class htmlTable
 #'
 #' @example inst/examples/htmlTable_example.R
@@ -253,8 +288,9 @@ htmlTable <- function(x, ...){
 
 `.` <- "magrittr RCM check issue"
 
-#' @importFrom stringr str_trim
-#' @importFrom stringr str_replace
+#' @importFrom stringr str_replace str_replace_all str_trim
+#' @importFrom htmltools htmlEscape
+#' @import checkmate
 #' @import magrittr
 #' @rdname htmlTable
 #' @export
@@ -312,8 +348,13 @@ htmlTable.default <- function(x,
                               ctable = TRUE,
                               compatibility = getOption("htmlTableCompat", "LibreOffice"),
                               cspan.rgroup = "all",
+                              escape.html = FALSE,
                               ...)
 {
+  if (isTRUE(escape.html)) {
+    x <- prEscapeHtml(x)
+  }
+
   if (is.null(dim(x))){
     if (!is.numeric(x) && !is.character(x)){
       x <- as.character(x)
@@ -326,32 +367,27 @@ htmlTable.default <- function(x,
          " length(dim(x)) = ", length(dim(x)) , " != 2")
   }
 
+
   if (missing(rgroup) &&
       !missing(n.rgroup)){
     # Add "" rgroups corresponding to the n.rgroups
     rgroup = rep("", length.out=length(n.rgroup))
   }
 
-  ## this will convert color names to hexadecimal (easier for user)
-  ## but also leaves hex format unchanged
-  col.rgroup <- prPrepareColors(col.rgroup, n = nrow(x), ng = n.rgroup, gtxt = rgroup)
-  col.columns <- prPrepareColors(col.columns, ncol(x))
-
   # Unfortunately in knitr there seems to be some issue when the
   # rnames is specified immediately as: rnames=rownames(x)
   if (missing(rnames)){
     if (any(is.null(rownames(x)) == FALSE))
       rnames <- rownames(x)
-  }
 
-  if (missing(rnames) &&
-        any(is.null(rownames(x))) &&
+    if (any(is.null(rownames(x))) &&
         !missing(rgroup)){
-    warning("You have not specified rnames but you seem to have rgroups.",
-            " If you have the first column as rowname but you want the rgroups",
-            " to result in subhedings with indentation below then, ",
-            " you should change the rnames to the first column and then",
-            " remove it from the table matrix (the x argument object).")
+      warning("You have not specified rnames but you seem to have rgroups.",
+              " If you have the first column as rowname but you want the rgroups",
+              " to result in subhedings with indentation below then, ",
+              " you should change the rnames to the first column and then",
+              " remove it from the table matrix (the x argument object).")
+    }
   }
 
   if (!missing(rowlabel) &&
@@ -392,13 +428,16 @@ htmlTable.default <- function(x,
       rgroup <- rgroup[n.rgroup >= 1]
       n.rgroup <- n.rgroup[n.rgroup >= 1]
     }
+
     # Sanity check for rgroup
     if (sum(n.rgroup) >  nrow(x)){
       stop("Your rows are fewer than suggested by the n.rgroup,",
            " i.e. ", sum(n.rgroup) , "(n.rgroup) > ", nrow(x), "(rows in x)")
-    }else if (sum(n.rgroup) < nrow(x) &&
-              (length(n.rgroup) == length(rgroup) - 1 ||
-               length(n.rgroup) == length(rgroup))){
+    }
+
+    if (sum(n.rgroup) < nrow(x) &&
+        (length(n.rgroup) == length(rgroup) - 1 ||
+         length(n.rgroup) == length(rgroup))){
       # Add an empty rgroup if missing
       if (length(n.rgroup) == length(rgroup))
         rgroup <- c(rgroup, "")
@@ -442,6 +481,11 @@ htmlTable.default <- function(x,
     cspan.rgroup <- rep(cspan.rgroup, length.out = length(rgroup))
   }
 
+  ## this will convert color names to hexadecimal (easier for user)
+  ## but also leaves hex format unchanged
+  col.rgroup <- prPrepareColors(col.rgroup, n = nrow(x), ng = n.rgroup, gtxt = rgroup)
+  col.columns <- prPrepareColors(col.columns, ncol(x))
+
   if (!missing(tspanner)){
 
     # Sanity checks css.tspanner and prepares the style
@@ -476,18 +520,87 @@ htmlTable.default <- function(x,
     }
   }
 
+  # Convert dimnames to something useful
+  if (!is.null(names(dimnames(x)))) {
+    # First dimname is always the variable name for the row
+    dimname4row <- names(dimnames(x))[1]
+    if (!is.null(dimname4row) && dimname4row != "") {
+      # Use rgroup or tspanner as this is visually more separated than rowlabel
+      # if these are available
+      if (missing(rgroup)) {
+        rgroup <- dimname4row
+        n.rgroup <- nrow(x)
+      } else if (missing(tspanner)) {
+        tspanner <- dimname4row
+        n.tspanner <- nrow(x)
+      } else if (missing(rowlabel)) {
+        rowlabel <- dimname4row
+      }
+    }
+
+    # Second dimname is always the variable name for the columns
+    dimname4col <- names(dimnames(x))[2]
+    if (!is.null(dimname4col) && dimname4col != "") {
+      # Use rgroup or tspanner as this is visually more separated than rowlabel
+      # if these are available
+      if (missing(cgroup)) {
+        cgroup <- dimname4col
+        n.cgroup <- ncol(x)
+
+        # If this is a addmargins object we shouldn't have the cspanner including the
+        # sum marker
+        if (!missing(total) && total &&
+            grepl("^sum$", tail(colnames(x), 1), ignore.case = TRUE)) {
+          cgroup %<>% c("")
+          n.cgroup <- c(n.cgroup[1] -1, 1)
+        }
+      }
+    }
+  }
 
   # Sanity check for tspanner
   if (!missing(tspanner)){
     if (missing(n.tspanner))
       stop("You need to specify the argument n.tspanner if you want to use table spanners")
 
-    if(sum(n.tspanner) !=  nrow(x))
-      stop(sprintf("Your rows don't match in the n.tspanner, i.e. %d != %d",
-                   sum(n.tspanner), nrow(x)))
+    if (any(n.tspanner < 1)) {
+      stop("You have  provided invalid number of rows in the n.tspanner argument - minimum is 1, you have: ",
+           vector2string(n.tspanner),
+           " where no. ", vector2string(which(n.tspanner)),
+           " was less than 1")
+    }
+    if (length(n.tspanner) == length(tspanner) - 1) {
+      if (missing(rgroup) || sum(n.tspanner) > length(rgroup)) {
+        n.tspanner = append(n.tspanner, nrow(x) - sum(n.tspanner))
+      } else {
+        n.tspanner = append(n.tspanner, length(rgroup) - sum(n.tspanner))
+      }
+    }
+    if (any(n.tspanner < 1)) {
+      stop("You have more tspannners than n.tspanner while the number of rows doesn't leave room for more tspanners")
+    }
+
+    if(sum(n.tspanner) !=  nrow(x)) {
+      if (missing(rgroup))
+        stop(sprintf("Your rows don't match in the n.tspanner, i.e. %d != %d",
+                     sum(n.tspanner), nrow(x)))
+
+      if (sum(n.tspanner) != length(rgroup))
+        stop(sprintf("Your rows don't match either the total number of rows '%d'
+                     or the number of rgroups '%d' the sum of n.tspanner %d",
+                     nrow(x),
+                     length(rgroup),
+                     sum(n.tspanner)))
+
+      org_nt <- n.tspanner
+      for (i in 1:length(n.tspanner)) {
+        offset <- sum(org_nt[0:(i-1)]) + 1
+        n.tspanner[i] = sum(n.rgroup[offset:(offset + org_nt[i] - 1)])
+      }
+    }
 
     # Make sure there are no collisions with rgrou
-    if (!missing(n.rgroup)){
+    if (!missing(n.rgroup)) {
       for (i in 1:length(n.tspanner)){
         rows <- sum(n.tspanner[1:i])
         if (!rows %in% cumsum(n.rgroup))
@@ -495,8 +608,8 @@ htmlTable.default <- function(x,
                tspanner[i],
                " (no. ", i, ") with rgroup splits.",
                " The missing row splitter should be on row number ", rows,
-               " and is not in the n.rgroup list: ", paste(n.rgroup, collapse=", "),
-               " note, it should match the cumulative sum n.rgroup", paste(cumsum(n.rgroup), collapse=", "))
+               " and is not in the n.rgroup list: ", vector2string(n.rgroup),
+               " note, it should match the cumulative sum n.rgroup", vector2string(cumsum(n.rgroup)))
       }
     }
   }
@@ -584,13 +697,17 @@ htmlTable.default <- function(x,
   }else{
     stop("You have provided an invalid 'total' argument:",
          " '", paste(total, collapse="', '"), "' ",
-         " of the class ", class(total), ".",
+         " of the class ", paste(class(total), collapse = " & "), ".",
          " The function currently only accepts logical or numerical",
          " values.")
   }
 
   css.total <- rep(css.total, length.out = length(total))
 
+  assert(
+    check_matrix(css.cell),
+    check_character(css.cell)
+  )
   css.cell <- prPrepareCss(x, css = css.cell,
                            rnames = rnames, header = header)
 
@@ -678,7 +795,7 @@ htmlTable.default <- function(x,
   tspanner_iterator <- 0
   if(nrow(x) > 0){
   for (row_nr in 1:nrow(x)){
-    rname_style = attr(css.cell, "rnames")[row_nr + !prSkipRownames(rnames)]
+    rname_style = attr(css.cell, "rnames")[row_nr]
 
     # First check if there is a table spanner that should be applied
     if (!missing(tspanner) &&
@@ -818,7 +935,7 @@ htmlTable.default <- function(x,
       table_str %<>%
         sprintf("%s\n\t\t<td style='%s'>%s%s</td>",
                 .,
-                prGetStyle(cell_style,
+                prGetStyle(c(rname_style, cell_style),
                              align=prGetAlign(align, 1)),
                 pdng,
                 rnames[row_nr])
@@ -878,9 +995,39 @@ htmlTable.default <- function(x,
   class(table_str) <- c("htmlTable", class(table_str))
   attr(table_str, "...") <- list(...)
 
+  # Add html class if this is a table inside a notebook for inline output
+  if (!getOption('htmlTable.skip_notebook', FALSE) && prIsNotebook()) {
+    class(table_str) <- c("html", class(table_str))
+    attr(table_str, "html") <- TRUE
+  }
   return(table_str)
 }
 
+#' Detects if the call is made from within an RStudio Rmd file or a file
+#' with the html_notebook output set.
+#' @importFrom rstudioapi isAvailable getActiveDocumentContext
+#' @keywords internal
+prIsNotebook <- function() {
+  if (!isAvailable()) {
+    return(FALSE)
+  }
+
+  ctxt <- getActiveDocumentContext()
+  if (grepl("\\.Rmd$", ctxt$path)) {
+    return(TRUE)
+  }
+
+  # Look for html_notebook within the header if the file hasn't been saved
+  contents <- ctxt$contents
+  header <- grep("^---$", contents)
+  if (length(header) == 2) {
+    return(any(grepl("html_notebook$",
+                     contents[min(header) : max(header)])))
+  }
+
+  return(FALSE)
+}
+
 #' Convert all factors to characters to print them as they expected
 #'
 #' @inheritParams htmlTable
@@ -918,12 +1065,20 @@ htmlTable.data.frame <- function(x, ...) {
 }
 
 #' @export
-htmlTable.matrix <- function(x, ...) {
-  # deal gracefully with an empty dataframe - issue a warning.
+htmlTable.matrix <- function(x, total, ...) {
+  # deal gracefully with an empty matrix - issue a warning.
   if(nrow(x) == 0){
     warning(paste(deparse(substitute(x)), "is an empty object"))
   }
-  htmlTable.default(x,...)
+
+  if (all(class(x) %in% c("table", "matrix", "array")) &&
+      !is.null(rownames(x)) &&
+      grepl("^sum$", tail(rownames(x), 1), ignore.case = TRUE) &&
+      missing(total)) {
+    total = TRUE
+  }
+
+  htmlTable.default(x, total = total, ...)
 }
 
 #' @importFrom methods setClass
@@ -943,9 +1098,9 @@ knit_print.htmlTable<- function(x, ...){
 #'  the table within that is envoced if in \code{\link[base]{interactive}} mode.
 #'  Set this to \code{FALSE} if you want to remove that  functionality. You can
 #'  also force the function to call a specific viewer by setting this to a
-#'  viewer function, e.g. \code{useViewer = utils::browseUrl} if you want to
+#'  viewer function, e.g. \code{useViewer = utils::browseURL} if you want to
 #'  override the default RStudio viewer. Another option that does the same is to
-#'  set the \code{options(viewer=utils::browseUrl)} and it will default to that
+#'  set the \code{options(viewer=utils::browseURL)} and it will default to that
 #'  particular viewer (this is how RStudio decides on a viewer).
 #'  \emph{Note:} If you want to force all output to go through the
 #'  \code{\link[base]{cat}()} the set \code{\link[base]{options}(htmlTable.cat = TRUE)}.
@@ -966,19 +1121,19 @@ print.htmlTable<- function(x, useViewer, ...){
   # useViewer parameter
   if (missing(useViewer)){
     if ("useViewer" %in% names(args) &&
-      (is.logical(args$useViewer) ||
+        (is.logical(args$useViewer) ||
          is.function(args$useViewer))){
-        useViewer <- args$useViewer
-        args$useViewer <- NULL
+      useViewer <- args$useViewer
+      args$useViewer <- NULL
     }else{
       useViewer <- TRUE
     }
   }
 
   if (interactive() &&
-        !getOption("htmlTable.cat", FALSE) &&
-        (is.function(useViewer) ||
-        useViewer != FALSE))
+      !getOption("htmlTable.cat", FALSE) &&
+      (is.function(useViewer) ||
+       useViewer != FALSE))
   {
     if (is.null(args$file)){
       args$file <- tempfile(fileext=".html")
@@ -990,7 +1145,7 @@ print.htmlTable<- function(x, useViewer, ...){
                       "</head>",
                       "<body>",
                       "<div style=\"margin: 0 auto; display: table; margin-top: 1em;\">",
-                      x,
+                      enc2utf8(x),
                       "</div>",
                       "</body>",
                       "</html>", sep="\n")
@@ -1006,7 +1161,7 @@ print.htmlTable<- function(x, useViewer, ...){
     }else{
       viewer <- getOption("viewer")
       if (!is.null(viewer) &&
-            is.function(viewer)){
+          is.function(viewer)){
         # (code to write some content to the file)
         viewer(args$file)
       }else{
@@ -1090,3 +1245,4 @@ tblNoNext <- function(roman = getOption("table_counter_roman",
 
   return(next_no)
 }
+
