@@ -37,3 +37,21 @@ test_that("Check that row styles are present",{
     }
   }
 })
+
+
+test_that("Check prPrepareCss",{
+  mx <-
+    matrix(1:5, ncol=5, nrow=1)
+  rownames(mx) <- "1st"
+  colnames(mx) <- paste(c("1st", "2nd", "3rd", paste0(4:ncol(mx), "th")), "hdr")
+
+  css.cell = rep("font-size: 1em", times = ncol(mx) + 1)
+  css.cell[1] = "font-size: 2em"
+  out <- prPrepareCss(mx, css = css.cell, header = names(mx), rnames = rownames(mx))
+  expect_equal(dim(out), dim(mx))
+
+
+  css.cell =  matrix("padding-left: .5em;", nrow = nrow(mx) + 1, ncol = ncol(mx))
+  out <- prPrepareCss(mx, css = css.cell, header=names(mx), rnames = rownames(mx))
+  expect_equal(dim(out), dim(mx))
+})
